//
//  sample-11_1.cpp
//  axis_histgrapm
//
//  Created by sawano on 2016/08/26.
//  Copyright © 2016年 AIT. All rights reserved.
//
#include <iostream>
#include <opencv2/opencv.hpp>

#define IMG1 "objectmove1.png"
#define IMG2 "objectmove2.png"

//ヒストグラムの初期化
void initHistgram (int *hist, int size) {
    int x;
    for (x=0; x<size; x++) {
        hist[x] = 0;
    }
}

//ヒストグラム計算 (縦方向)
void createHistgramCols (cv::Mat src_img, int *hist) {
    int x, y;
    unsigned char v; //値
    //画像の走査
    for (y=0; y<src_img.rows; y++) {
        for (x=0; x<src_img.cols; x++) {
            v = src_img.at<unsigned char>(y, x);
            if (v==0) { //物体 (黒) が存在するとき
                hist[x]++; //度数の加算
            }
        }
    }
}

//物体の位置 (開始位置)
int calcPoistionHorizontal (int *hist, int size) {
    int x;
    for (x=0; x<size; x++) {
        if (hist[x] > 0) {
            break;
        }
    }
    printf ("x = %d\n", x);
    return (x);
}

int main(int argc, const char * argv[]) {
    //画像の入力 (グレースケール)
    cv::Mat src_img1 = cv::imread(IMG1, 0);
    cv::Mat src_img2 = cv::imread(IMG2, 0);
    
    //ニ値画像のメモリ確保
    cv::Mat bin_img1, bin_img2;
    
    //画像が入力できているか確認
    if (src_img1.empty() && src_img2.empty()) { //入力失敗の場合
        fprintf(stderr, "File is not opened.\n");
        return (-1);
    }
    //ニ値化
    cv::threshold(src_img1, bin_img1, 0, 255, cv::THRESH_BINARY | cv::THRESH_OTSU);
    cv::threshold(src_img2, bin_img2, 0, 255, cv::THRESH_BINARY | cv::THRESH_OTSU);
    
    //ヒストグラムの生成 (横方向のみ)
    int hist1[src_img1.cols], hist2[src_img2.cols];
    initHistgram(hist1, src_img1.cols); //ヒストグラムの初期化
    initHistgram(hist2, src_img2.cols); //ヒストグラムの初期化
    
    createHistgramCols(bin_img1, hist1); //ヒストグラムの計算
    createHistgramCols(bin_img2, hist2); //ヒストグラムの計算
    
    printf("移動距離=%d\n", calcPoistionHorizontal(hist1, src_img1.cols)-calcPoistionHorizontal(hist2, src_img2.cols));
    
    return 0;
}
