//
//  main.cpp
//  labeling_basic
//
//  Created by Masashi Morimoto on 2016/01/29.
//  Copyright © 2016年 AIT. All rights reserved.
//

/*** ラベリング処理のプログラム ***/
#include <iostream>
#include <opencv2/opencv.hpp>

#define HIGHVAL (255)   // 白画素の値
#define FILENAME "/Users/foobar/Pictures/figures_binarized.png" // 対象二値画像
#define NOLABEL (0)         // ラベルなしの値
#define MAXLABEL (65535)    // ラベル番号の上限値
#define NEIGHBORS (4)       // 近傍画素数（4近傍の場合）
int delta[NEIGHBORS][2] ={{0,-1},{-1,0},{1,0},{0,1}};
                            // 注目画素から見た近傍画素の相対位置
int main(int argc, const char * argv[])
{
    // 変数定義
    cv::Mat bin_img, labeled_img, result_img;
    int nx, ny;
    unsigned int currentLabel, neighborLabel, minNeighborLabel, labelNo = 0;
    bool isUpdated;
    
    // 画像入力
    bin_img = cv::imread(FILENAME, cv::IMREAD_GRAYSCALE);
    if (bin_img.empty()) {  // 画像ファイル読み込み失敗
        printf("Cannot read image file: %s\n", FILENAME);
        return (-1);
    }

    // ラベル画像の生成 （初期値はラベルなし）
    labeled_img.create(bin_img.size(), CV_16UC1); labeled_img.setTo(NOLABEL);
    // 更新がなくなるまで走査処理を繰り返す
    do {
        isUpdated = false;
        for (int y=0; y<bin_img.rows; y++) {
            for (int x=0; x<bin_img.cols; x++) {
                if (bin_img.at<unsigned char>(y,x) == HIGHVAL) {    // 注目画素が白画素の場合
                    currentLabel = labeled_img.at<uint16_t>(y,x);   // 注目画素の現ラベル値を取得（１回目はNOLABEL）
                    minNeighborLabel = MAXLABEL;                    // 最小近傍ラベル値の初期値設定（上限値）
                    for (int idxN=0; idxN<NEIGHBORS; idxN++) {  // 近傍画素を探索する
                        nx = x + delta[idxN][0]; ny = y + delta[idxN][1];
                        if (nx >= 0 && nx < labeled_img.cols && ny >= 0 && ny < labeled_img.rows) {
                            neighborLabel = labeled_img.at<uint16_t>(ny,nx);
                            if (neighborLabel != NOLABEL && neighborLabel < minNeighborLabel) {
                                minNeighborLabel = neighborLabel;   // 近傍画素のラベル値がこれまでの最小ならば更新
                            }
                        }
                    }
                    if (currentLabel == NOLABEL) {              // 注目画素のラベルがない場合（１回目）
                        if (minNeighborLabel < MAXLABEL) {          // 近傍画素にラベル値があれば
                            labeled_img.at<uint16_t>(y,x) = minNeighborLabel;   // 最小近傍ラベル値に更新
                        } else {                                    // 近傍画素にラベル値がなければ
                            labeled_img.at<uint16_t>(y,x) = ++labelNo;          // 新規ラベルを注目画素に付与
                        }
                        isUpdated = true;                           // 更新フラグを立てる
                    } else {                                    // 注目画素のラベルがある場合（２回目以降）
                        if (minNeighborLabel < currentLabel) {      // 現ラベルより近傍最小ラベル値が小さければ
                            labeled_img.at<uint16_t>(y,x) = minNeighborLabel;   // 最小近傍ラベル値に更新
                            isUpdated = true;                       // 更新フラグを立てる                        }
                        }
                    }
                }                                       // 黒画素の場合は何もしない
            }
        }
    } while (isUpdated);

    // 特定ラベルの画素領域だけ描画
    result_img.create(bin_img.size(), bin_img.type());
    for (int y=0; y<result_img.rows; y++) {
        for (int x=0; x<result_img.cols; x++) {
            currentLabel = labeled_img.at<uint16_t>(y,x);
            if (currentLabel == 0) {
                result_img.at<unsigned char>(y,x)=HIGHVAL;
            }
        }
    }
    //画像表示
    cv::imshow("input image", bin_img);
    cv::imshow("labeled image", result_img);
    //キー入力待ち
    cv::waitKey(0);
    return (0);
}
