//
//  main.cpp
//  HoughLines_basic
//
//  Created by Masashi Morimoto on 2016/03/06.
//  Copyright © 2016年 AIT. All rights reserved.
//

/*** 直線抽出処理のプログラム ***/
#include <iostream>
#include <opencv2/opencv.hpp>

#define FILENAME    "/Users/foobar/Pictures/room.png" // 対象画像
#define RHOSTEP (1.0)   // 距離ρの刻み幅（画素）
#define THETASTEP (1.0) // 角度θの刻み幅（度）
#define TH (150)        // 直線抽出のしきい値

int main(int argc, const char * argv[])
{
    // 変数定義
    cv::Mat gray_img, edge_img, houghPlane;
    double width, height, maxLen, rho, theta;
    int rhoMax, thetaMax;

    // 画像入力およびエッジ検出
    gray_img = cv::imread(FILENAME, cv::IMREAD_GRAYSCALE);
    if (gray_img.empty()) { // 画像ファイル読み込み失敗
        printf("Cannot read image file: %s\n", FILENAME);
        return (-1);
    }
    cv::Canny(gray_img, edge_img, 50.0, 150.0);
    
    // 直線パラメータ空間確保
    width = edge_img.cols; height = edge_img.rows;
    maxLen = sqrt(width*width + height*height);     // 線分がとりうる原点からの最大距離
    rhoMax = ceil(maxLen/RHOSTEP); thetaMax = ceil(360/THETASTEP);  // パラメータ空間のセル数
    houghPlane = cv::Mat::zeros(thetaMax, rhoMax, CV_16UC1);    // パラメータ空間確保
    // 投票処理
    for (int y=0; y < height; y++) {
        for (int x=0; x < width; x++) {
            unsigned char pix = edge_img.at<unsigned char>(y, x);
            if (pix > 0) {                          // 画素がある場合は投票を行う
                for (int thetaIdx = 0; thetaIdx < thetaMax; thetaIdx++) { // 角度セルごとに
                    theta = thetaIdx*THETASTEP*CV_PI/180;                   // 角度（ラジアン）
                    rho = x*cos(theta) + y*sin(theta);                      // 原点からの距離を算出
                    int rhoIdx = round(rho/RHOSTEP);
                    houghPlane.at<uint16_t>(thetaIdx, rhoIdx)++;            // 該当するセルに投票
                }
            }
        }
    }
    // 多数決処理
    for (int thetaIdx = 0; thetaIdx < thetaMax; thetaIdx++) {
        for (int rhoIdx = 0; rhoIdx < rhoMax; rhoIdx++) {
            uint16_t cell = houghPlane.at<uint16_t>(thetaIdx, rhoIdx);
            if (cell > TH) {                                        // 投票数がしきい値を超えた場合
                rho = rhoIdx*RHOSTEP;                               // 距離ρ
                theta = thetaIdx*THETASTEP;                         // 角度θ（ここでは度）を抽出できる
                printf("(theta,rho)=(%.1f,%.1f)\n", theta, rho);
            }
        }
    }
    return (0);
}

