//
//  sample10-1.cpp
//  Stereo画像処理
//
//  Created by Tsutomu Horikoshi on 2016/11/13.
//  Copyright © 2016年 SIT. All rights reserved.
//

/***ステレオ画像の対応点探索プログラム***/

#include <iostream>
#include <opencv2/opencv.hpp>

// 画像ファイル名
#define left_FILENAME "Users/foobar/Pictures/stereoL.jpg"
#define right_FILENAME "Users/foobar/Pictures/stereoR.jpg"

int main(int argc, const char ** argv)
{
	//探索位置座標　例えば(300，300)を設定	
	int x0=300, y0=300; 
	
	//探索ウィンドウ（サイズ 縦9画素，横9画素）を設定する
	int win_size=9,win_half=win_size/2;
	//類似度の値を入れる配列を定義する
	int similarity[900];
	
	// 画像変数の定義
	cv::Mat left_img, right_img;
	
	// 2枚の画像を読み込む
	left_img = cv::imread(left_FILENAME, cv::IMREAD_GRAYSCALE );
	if( left_img.empty()){ //画像ファイル読み込み失敗
		printf("Cannot read image file: %s\n", left_FILENAME);		
		return (-1);
	}	
	right_img = cv::imread(right_FILENAME, cv::IMREAD_GRAYSCALE );
	if( right_img.empty()){ //画像ファイル読み込み失敗
		printf("Cannot read image file: %s\n", right_FILENAME);		
		return (-1);
	}	

	
	//エピポーラライン（ｙ座標＝y0）に沿って探索ウィンドウを移動させる
	for(int x=win_half;x<right_img.cols-win_half;x++){
		int diff=0;
		int sum=0;
		for(int sy=0;sy<win_size;sy++){
			for(int sx=0;sx<win_size;sx++){
				//テンプレートと探索ウィンドウの画素値の類似度を算出する
				int intensity = right_img.at<uchar>(sy+y0-win_half,sx+x-win_half);
				diff += std::fabs(left_img.at<uchar>(sy+y0-win_half,sx+x0-win_half)-intensity);
				sum += left_img.at<uchar>(sy+y0-win_half,sx+x-win_half);				
			}
		}
		//類似度の配列Similarityに記録する
		similarity[x]=diff;		
	}
	//配列Similarityに記録された類似度の最小値を探す 
	int minimum=similarity[win_half];
	int min_num=win_half;
	for(int x=win_half;x<left_img.cols-win_half;x++){
		if(minimum > similarity[x]) {
			minimum = similarity[x];
			min_num=x;
		}
	}
	
	//検出された最小値min_mumが2つの画像の対応点の座標となる
	cv::circle(left_img, cv::Point(x0, y0),10, cv::Scalar(255,255,255),2);
	cv::circle(right_img, cv::Point(min_num, y0),10, cv::Scalar(255,255,255),2);
	cv::imshow("Left",left_img);
	cv::imshow("Right",right_img);
	cv::imwrite("LeftResult.png",left_img);
	cv::imwrite("RightResult.png",right_img);
	
	cv::waitKey(0);
	return (0);
}
