//
//  sample6-1.cpp
//  離散フーリエ変換
//
//  Reference http://docs.opencv.org/2.4/doc/tutorials/core/discrete_fourier_transform/discrete_fourier_transform.html
//  Revised by Tsutomu Horikoshi on 2016/11/13.
//  Copyright © 2016年 SIT. All rights reserved.
//
/***離散フーリエ変換のプログラム***/

#include <iostream>
#include <opencv2/opencv.hpp>

#define FILENAME "Users/foobar/Pictures/lenna.jpg"

int main(int argc, char * argv[])
{
	//変数定義
	cv::Mat src_img, comp_img, tmp;    
	
	//画像入力
	src_img = cv::imread(FILENAME, cv::IMREAD_GRAYSCALE);
	if( src_img.empty()){ //画像ファイル読み込み失敗
		printf("Cannot read image file: %s\n", FILENAME);		
		return (-1);
	}	
	
	//sec_imgと同じサイズで，要素の値が全てゼロの配列を用意する
	cv::Mat planes[] = {cv::Mat_<float>(src_img), cv::Mat::zeros(src_img.size(), CV_32F)};
	
	//２つの配列planesを結合し，複素数の配列comp_imgとする
	cv::merge(planes, 2, comp_img);         
	//フーリエ変換を実施
	cv::dft(comp_img, comp_img);            // this way the result may fit in the source cv::Matrix
	
	//複素数配列を分解　planes[0] = 実数部(comp_img), planes[1] = 虚数部(comp_img)
	cv::split(comp_img, planes);  
	
	//振幅を算出し，パワースペクトルラム（pow_spc)に代入する
	cv::magnitude(planes[0], planes[1], planes[0]);
	cv::Mat pow_spc = planes[0];
	
	// パワースペクトルの値を対数スケールに変換　
	// => log(1 + sqrt(Re(DFT(I))^2 + Im(DFT(I))^2))
	pow_spc += cv::Scalar::all(1);     
	
	//pow_spcの画像をlogスケールに変換する
	cv::log(pow_spc, pow_spc);
	
	//  光学フーリエ変換のパワースペクトルに変換
	int cx = pow_spc.cols/2;
	int cy = pow_spc.rows/2;
	
	cv::Mat q0(pow_spc, cv::Rect(0, 0, cx, cy));   // 第2象限
	cv::Mat q1(pow_spc, cv::Rect(cx, 0, cx, cy));  // 第1象限
	cv::Mat q2(pow_spc, cv::Rect(0, cy, cx, cy));  // 第3象限
	cv::Mat q3(pow_spc, cv::Rect(cx, cy, cx, cy)); // 第4象限
	
	// 各象限の値を入れ替える
	q0.copyTo(tmp);
	q3.copyTo(q0);
	tmp.copyTo(q3);
	
	q1.copyTo(tmp);         
	q2.copyTo(q1);
	tmp.copyTo(q2);
	
	//画像として表示できる値にパワースペクトラムの値を正規化する
	cv::normalize(pow_spc, pow_spc, 0, 1, cv::NORM_MINMAX); 
	
	cv::imshow("Input Image" , src_img   );  
	cv::imshow("spectrum magnitude", pow_spc);
	cv::imwrite( "result.bmp", pow_spc );
	
	cv::waitKey(0);
	
	return 0;
}